library(foreign)
library(nnet)
library(MASS)
library(ggplot2)
library(reshape2)


#We have attendance data on 314 high school juniors from two urban high schools in 
#the file nb_data. The response variable of interest is days absent, daysabs. 
#The variable math gives the standardized math score for each student. 
#The variable prog is a three-level nominal variable indicating the type of instructional 
#program in which the student is enrolled.

dat <- read.dta("https://stats.idre.ucla.edu/stat/stata/dae/nb_data.dta")
dat <- within(dat, {
  prog <- factor(prog, levels = 1:3, labels = c("General", "Academic", "Vocational"))
  id <- factor(id)
})


with(dat, tapply(daysabs, prog, function(x) {
  sprintf("M (SD) = %1.2f (%1.2f)", mean(x), sd(x))
}))

# Analysis methods we might consider:


#Poisson regression – Poisson regression is often used for modeling count data.

PoissonModel<- glm(daysabs ~ math + prog, data = dat, family=poisson(link = "log"))
summary(PoissonModel)

#Intepretation of the coefficients:
#Math: for each one-unit increase in math, the expected log count of the number 
#of days absent decreases by 0.006. 

# progAcademic: is the expected difference in log count between group 2 and the reference 
#group (prog=1). The expected log count for level 2 of prog is 0.44 lower than 
#the expected log count for level 1 (General). 

# progVocational: is the expected difference in log count between group 3 and the 
#reference group.The expected log count for level 3 of prog is 1.28 lower than the
#expected log count for level 1 (General). 

#McFadden’s R-Squared:
1-PoissonModel$deviance/PoissonModel$null.deviance

#Estimates seems to be quite good: all the estimated parameters are significant. 
# We may be in two scenarios:
#1) The model perfectly fits the data and estimates are reliable
#2) There is overdispersion: the standard errors of the Poisson model parameters 
# are underestimated and coefficients result sifnificant.

#Deviance of the residuals vs df
#As a rule of thumb if this ratio is > 1, data are overdispersed: 
PoissonModel$deviance/PoissonModel$df.residual


#Negative binomial regression -Negative binomial regression  (used for over-dispersed 
#count data, that is when the conditional variance exceeds the conditional mean)

library(MASS)
negBinModel <- glm.nb(daysabs ~ math + prog, data = dat)
summary(negBinModel)

#The dispersion parameter is Theta
negBinModel$theta

# To determine if prog itself, overall, is statistically significant, we can compare 
# a model with and without prog via likelihood-ratio test.

negBinModel2 <- glm.nb(formula = daysabs ~ math, data = dat, link = log)
anova(negBinModel, negBinModel2)

###Or alternatively
negBinModel2 <- update(negBinModel, . ~ . - prog)
pchisq(2 * (logLik(negBinModel) - logLik(negBinModel2)), df = 2, lower.tail = FALSE)
# The Likelihood ratio test indicates that prog is a statistically
# significant predictor of daysabs.
# The null deviance is calculated from an intercept-only model with 313 degrees of freedom.
# Then we see the residual deviance, the deviance from the full model. 
# We are also shown the AIC and 2*log likelihood.
# The theta parameter shown is the dispersion parameter. 


# The Poisson model is actually nested in the negative binomial model. 
# We can then use a likelihood ratio test to compare these two and test this model assumption.

#### Comparing negative Binomial and Poisson model ###
### We compare the two models with a Likelihood ratio - test:
pchisq(2 * (logLik(negBinModel) - logLik(PoissonModel)), df = 1, lower.tail = FALSE)

#In this example the associated chi-squared value estimated 
#from 2*(logLik(negBinModel) – logLik(PoissonModel)) is 926.03 with one degree of freedom. 
#This strongly suggests the negative binomial model, estimating the dispersion parameter, 
#is more appropriate than the Poisson model.


#Alternative model for overdispersed data: Quasi-Poisson regression.


QuasiPoissonModel<- glm(daysabs ~ math + prog, data = dat, family=quasipoisson(link = "log"))
summary(QuasiPoissonModel)
